@@ -553,6 +553,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     int _i_sqrt_minus1_;
   };
   std::string gen2string(const gen & g);
+  const int turtle_length=10;
 #ifdef KHICAS
   struct logo_turtle {
     double x,y;
@@ -560,13 +561,13 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     bool visible; // true if turtle visible
     bool mark; // true if moving marks
     bool direct; // true if rond/disque is done in the trigonometric direction
+    char turtle_width;
+    short int s;//std::string s;
     int color;
-    int turtle_length;
     int radius; // 0 nothing, >0 -> draw a plain disk 
     // bit 0-8=radius, bit9-17 angle1, bit 18-26 angle2, bit 27=1 filled  or 0 
     // <0 fill a polygon from previous turtle positions
-    int s;//std::string s;
-    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0) {}
+    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0) {}
   };
 #else // KHICAS
   struct logo_turtle {
@@ -576,16 +577,16 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     bool mark; // true if moving marks
     bool direct; // true if rond/disque is done in the trigonometric direction
     int color;
-    int turtle_length;
+    int turtle_width;
     int radius; // 0 nothing, >0 -> draw a plain disk 
     // bit 0-8=radius, bit9-17 angle1, bit 18-26 angle2, bit 27=1 filled  or 0 
     // <0 fill a polygon from previous turtle positions
     std::string s;
     void * widget;
 #ifdef IPAQ
-    logo_turtle(): x(70),y(70),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0),widget(0) {}
+    logo_turtle(): x(70),y(70),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0),widget(0) {}
 #else
-    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_length(10),radius(0),widget(0) {}
+    logo_turtle(): x(100),y(100),theta(0),visible(true),mark(true),direct(true),color(0),turtle_width(1),radius(0),widget(0) {}
 #endif
   };
 #endif // KHICAS
